/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

/**
 * An interface to individual entries in session history. Each 
 * document or frame will have a nsIHistoryEntry associated with 
 * it. nsIHistoryEntry provides access to information like URI, 
 * title and frame traversal status for that document.
 * This interface is accessible from javascript.
 */
#include "nsISupports.idl"

interface nsIURI;

[scriptable, uuid(A41661D4-1417-11D5-9882-00C04FA02F40)]
interface nsIHistoryEntry : nsISupports 
{

    /** 
     * A readonly property that returns the URI
     * of the current entry. The object returned is
     * of type nsIURI
     */
    readonly attribute nsIURI URI;

    /** 
     * A readonly property that returns the title
     * of the current entry.  The object returned
     * is a encoded string
     */
    readonly attribute wstring title;

    /** 
     * A readonly property that returns a boolean
     * flag which indicates if the entry was created as a 
     * result of a subframe navigation. This flag will be
     * 'false' when a frameset page is visited for
     * the first time. This flag will be 'true' for all
     * history entries created as a result of a subframe
     * navigation.
     */
    readonly attribute boolean isSubFrame;

};


%{ C++
// {A41661D5-1417-11D5-9882-00C04FA02F40}
#define NS_HISTORYENTRY_CID \
{0xa41661d5, 0x1417, 0x11d5, {0x98, 0x82, 0x0, 0xc0, 0x4f, 0xa0, 0x2f, 0x40}}

#define NS_HISTORYENTRY_CONTRACTID \
    "@mozilla.org/browser/history-entry;1"

%}

