/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMElement.idl"
#include "nsIDOMXULElement.idl"

interface nsIControllers;

[scriptable, uuid(3b1bce24-c3ad-4568-8e10-85922b521e8b)]
interface nsIDOMXULControlElement : nsIDOMXULElement {
  attribute boolean disabled;
  attribute long tabIndex;
  
// XXX defined in XULElement, but should be defined here
//  readonly attribute nsIControllers controllers;
  
//  void focus();
//  void blur();
};


