/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMDOMRequest.idl"

[scriptable, builtinclass, uuid(d77cafb5-7a91-4631-9597-5e4dcc9d90a5)]
interface nsIDOMMozSmsRequest : nsIDOMDOMRequest
{
};
