/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "domstubs.idl"

[scriptable, uuid(f6134683-f28b-11d2-8360-c90899049c3c)]
interface nsIDOMMimeTypeArray : nsISupports
{
  readonly attribute unsigned long    length;

  nsIDOMMimeType            item(in unsigned long index);
  nsIDOMMimeType            namedItem(in DOMString name);
};
